@@ -7,15 +7,16 @@ use squawk_syntax::{SyntaxNodePtr, ast, ast::AstNode};
 use crate::scope::{Scope, ScopeId};
 use crate::symbols::{Name, Schema, Symbol, SymbolKind};
 
-pub(crate) struct SearchPathChange {
+struct SearchPathChange {
     position: TextSize,
     search_path: Vec<Schema>,
 }
 
 pub(crate) struct Binder {
-    pub(crate) scopes: Arena<Scope>,
-    pub(crate) symbols: Arena<Symbol>,
-    pub(crate) search_path_changes: Vec<SearchPathChange>,
+    // TODO: doesn't seem like we need this with our resolve setup
+    scopes: Arena<Scope>,
+    symbols: Arena<Symbol>,
+    search_path_changes: Vec<SearchPathChange>,
 }
 
 impl Binder {
@@ -27,12 +28,16 @@ impl Binder {
             symbols: Arena::new(),
             search_path_changes: vec![SearchPathChange {
                 position: TextSize::from(0),
-                search_path: vec![Schema::new("public"), Schema::new("pg_temp")],
+                search_path: vec![
+                    Schema::new("public"),
+                    Schema::new("pg_temp"),
+                    Schema::new("pg_catalog"),
+                ],
             }],
         }
     }
 
-    pub(crate) fn root_scope(&self) -> ScopeId {
+    fn root_scope(&self) -> ScopeId {
         self.scopes
             .iter()
             .next()
@@ -40,6 +45,127 @@ impl Binder {
             .expect("root scope must exist")
     }
 
+    pub(crate) fn lookup(&self, name: &Name, kind: SymbolKind) -> Option<SyntaxNodePtr> {
+        let symbols = self.scopes[self.root_scope()].get(name)?;
+        let symbol_id = symbols.iter().copied().find(|id| {
+            let symbol = &self.symbols[*id];
+            symbol.kind == kind
+        })?;
+        Some(self.symbols[symbol_id].ptr)
+    }
+
+    pub(crate) fn lookup_with(
+        &self,
+        name: &Name,
+        kind: SymbolKind,
+        position: TextSize,
+        schema: &Option<Schema>,
+    ) -> Option<SyntaxNodePtr> {
+        let symbols = self.scopes[self.root_scope()].get(name)?;
+
+        let search_paths = match schema {
+            Some(s) => std::slice::from_ref(s),
+            None => self.search_path_at(position),
+        };
+
+        for search_schema in search_paths {
+            if let Some(symbol_id) = symbols.iter().copied().find(|id| {
+                let symbol = &self.symbols[*id];
+                symbol.kind == kind && symbol.schema.as_ref() == Some(search_schema)
+            }) {
+                return Some(self.symbols[symbol_id].ptr);
+            }
+        }
+        None
+    }
+
+    pub(crate) fn lookup_with_params(
+        &self,
+        name: &Name,
+        kind: SymbolKind,
+        position: TextSize,
+        schema: &Option<Schema>,
+        params: Option<&[Name]>,
+    ) -> Option<SyntaxNodePtr> {
+        let symbols = self.scopes[self.root_scope()].get(name)?;
+
+        let search_paths = match schema {
+            Some(s) => std::slice::from_ref(s),
+            None => self.search_path_at(position),
+        };
+
+        for search_schema in search_paths {
+            if let Some(symbol_id) = symbols.iter().copied().find(|id| {
+                let symbol = &self.symbols[*id];
+                let params_match = match (&symbol.params, params) {
+                    (Some(sym_params), Some(req_params)) => sym_params.as_slice() == req_params,
+                    (None, None) => true,
+                    (_, None) => true,
+                    _ => false,
+                };
+                symbol.kind == kind && symbol.schema.as_ref() == Some(search_schema) && params_match
+            }) {
+                return Some(self.symbols[symbol_id].ptr);
+            }
+        }
+        None
+    }
+
+    pub(crate) fn lookup_with_table(
+        &self,
+        name: &Name,
+        kind: SymbolKind,
+        position: TextSize,
+        schema: &Option<Schema>,
+        table: &Option<Name>,
+    ) -> Option<SyntaxNodePtr> {
+        let symbols = self.scopes[self.root_scope()].get(name)?;
+
+        let search_paths = match schema {
+            Some(s) => std::slice::from_ref(s),
+            None => self.search_path_at(position),
+        };
+
+        for search_schema in search_paths {
+            if let Some(symbol_id) = symbols.iter().copied().find(|id| {
+                let symbol = &self.symbols[*id];
+                symbol.kind == kind
+                    && symbol.schema.as_ref() == Some(search_schema)
+                    && &symbol.table == table
+            }) {
+                return Some(self.symbols[symbol_id].ptr);
+            }
+        }
+        None
+    }
+
+    pub(crate) fn lookup_info(
+        &self,
+        name_str: String,
+        schema: &Option<String>,
+        kind: SymbolKind,
+        position: TextSize,
+    ) -> Option<(Schema, String)> {
+        let name_normalized = Name::from_string(name_str.clone());
+        let symbols = self.scopes[self.root_scope()].get(&name_normalized)?;
+
+        let search_paths = match schema {
+            Some(schema_name) => &[Schema::new(schema_name)],
+            None => self.search_path_at(position),
+        };
+
+        for search_schema in search_paths {
+            if let Some(symbol_id) = symbols.iter().copied().find(|id| {
+                let symbol = &self.symbols[*id];
+                symbol.kind == kind && symbol.schema.as_ref() == Some(search_schema)
+            }) {
+                let symbol = &self.symbols[symbol_id];
+                return Some((symbol.schema.clone()?, name_str));
+            }
+        }
+        None
+    }
+
     fn current_search_path(&self) -> &[Schema] {
         &self
             .search_path_changes
@@ -59,6 +185,51 @@ impl Binder {
         // default search path
         &self.search_path_changes[0].search_path
     }
+
+    pub(crate) fn all_symbols_by_kind(
+        &self,
+        kind: SymbolKind,
+        schema: Option<&Schema>,
+    ) -> Vec<&Name> {
+        let root_scope = self.root_scope();
+        let scope = &self.scopes[root_scope];
+
+        let mut names = vec![];
+        for (name, symbol_ids) in &scope.entries {
+            for symbol_id in symbol_ids {
+                let symbol = &self.symbols[*symbol_id];
+                if symbol.kind == kind
+                    && schema.is_none_or(|schema| symbol.schema.as_ref() == Some(schema))
+                {
+                    names.push(name);
+                    break;
+                }
+            }
+        }
+        names
+    }
+
+    pub(crate) fn functions_with_single_param(&self, param_type: &Name) -> Vec<&Name> {
+        let root_scope = self.root_scope();
+        let scope = &self.scopes[root_scope];
+
+        let mut names = vec![];
+        for (name, symbol_ids) in &scope.entries {
+            for symbol_id in symbol_ids {
+                let symbol = &self.symbols[*symbol_id];
+                if symbol.kind == SymbolKind::Function
+                    && symbol
+                        .params
+                        .as_ref()
+                        .is_some_and(|p| p.len() == 1 && &p[0] == param_type)
+                {
+                    names.push(name);
+                    break;
+                }
+            }
+        }
+        names
+    }
 }
 
 pub(crate) fn bind(file: &ast::SourceFile) -> Binder {
@@ -79,7 +250,7 @@ fn bind_stmt(b: &mut Binder, stmt: ast::Stmt) {
     match stmt {
         ast::Stmt::CreateTable(create_table) => bind_create_table(b, create_table),
         ast::Stmt::CreateForeignTable(create_foreign_table) => {
-            bind_create_foreign_table(b, create_foreign_table)
+            bind_create_table(b, create_foreign_table)
         }
         ast::Stmt::CreateIndex(create_index) => bind_create_index(b, create_index),
         ast::Stmt::CreateFunction(create_function) => bind_create_function(b, create_function),
@@ -92,17 +263,26 @@ fn bind_stmt(b: &mut Binder, stmt: ast::Stmt) {
             bind_create_materialized_view(b, create_view)
         }
         ast::Stmt::CreateSequence(create_sequence) => bind_create_sequence(b, create_sequence),
+        ast::Stmt::CreateTrigger(create_trigger) => bind_create_trigger(b, create_trigger),
+        ast::Stmt::CreateEventTrigger(create_event_trigger) => {
+            bind_create_event_trigger(b, create_event_trigger)
+        }
         ast::Stmt::CreateTablespace(create_tablespace) => {
             bind_create_tablespace(b, create_tablespace)
         }
         ast::Stmt::CreateDatabase(create_database) => bind_create_database(b, create_database),
         ast::Stmt::CreateServer(create_server) => bind_create_server(b, create_server),
+        ast::Stmt::CreateExtension(create_extension) => bind_create_extension(b, create_extension),
+        ast::Stmt::CreateRole(create_role) => bind_create_role(b, create_role),
+        ast::Stmt::Declare(declare) => bind_declare_cursor(b, declare),
+        ast::Stmt::Prepare(prepare) => bind_prepare(b, prepare),
+        ast::Stmt::Listen(listen) => bind_listen(b, listen),
         ast::Stmt::Set(set) => bind_set(b, set),
         _ => {}
     }
 }
 
-fn bind_create_table(b: &mut Binder, create_table: ast::CreateTable) {
+fn bind_create_table(b: &mut Binder, create_table: impl ast::HasCreateTable) {
     let Some(path) = create_table.path() else {
         return;
     };
@@ -118,35 +298,22 @@ fn bind_create_table(b: &mut Binder, create_table: ast::CreateTable) {
     let table_id = b.symbols.alloc(Symbol {
         kind: SymbolKind::Table,
         ptr: name_ptr,
-        schema: Some(schema),
+        schema: Some(schema.clone()),
         params: None,
+        table: None,
     });
 
-    let root = b.root_scope();
-    b.scopes[root].insert(table_name, table_id);
-}
-
-fn bind_create_foreign_table(b: &mut Binder, create_foreign_table: ast::CreateForeignTable) {
-    let Some(path) = create_foreign_table.path() else {
-        return;
-    };
-    let Some(table_name) = item_name(&path) else {
-        return;
-    };
-    let name_ptr = path_to_ptr(&path);
-    let Some(schema) = schema_name(b, &path, false) else {
-        return;
-    };
-
-    let table_id = b.symbols.alloc(Symbol {
-        kind: SymbolKind::Table,
+    let type_id = b.symbols.alloc(Symbol {
+        kind: SymbolKind::Type,
         ptr: name_ptr,
         schema: Some(schema),
         params: None,
+        table: None,
     });
 
     let root = b.root_scope();
-    b.scopes[root].insert(table_name, table_id);
+    b.scopes[root].insert(table_name.clone(), table_id);
+    b.scopes[root].insert(table_name, type_id);
 }
 
 fn bind_create_index(b: &mut Binder, create_index: ast::CreateIndex) {
@@ -166,6 +333,7 @@ fn bind_create_index(b: &mut Binder, create_index: ast::CreateIndex) {
         ptr: name_ptr,
         schema: Some(schema),
         params: None,
+        table: None,
     });
 
     let root = b.root_scope();
@@ -194,6 +362,7 @@ fn bind_create_function(b: &mut Binder, create_function: ast::CreateFunction) {
         ptr: name_ptr,
         schema: Some(schema),
         params,
+        table: None,
     });
 
     let root = b.root_scope();
@@ -222,6 +391,7 @@ fn bind_create_aggregate(b: &mut Binder, create_aggregate: ast::CreateAggregate)
         ptr: name_ptr,
         schema: Some(schema),
         params,
+        table: None,
     });
 
     let root = b.root_scope();
@@ -250,6 +420,7 @@ fn bind_create_procedure(b: &mut Binder, create_procedure: ast::CreateProcedure)
         ptr: name_ptr,
         schema: Some(schema),
         params,
+        table: None,
     });
 
     let root = b.root_scope();
@@ -261,13 +432,9 @@ fn bind_create_schema(b: &mut Binder, create_schema: ast::CreateSchema) {
         let schema_name = Name::from_node(&schema_name_node);
         let name_ptr = SyntaxNodePtr::new(schema_name_node.syntax());
         (schema_name, name_ptr)
-    } else if let Some(schema_name_ref) = create_schema
-        .schema_authorization()
-        .and_then(|authorization| authorization.role())
-        .and_then(|role| role.name_ref())
-    {
-        let schema_name = Name::from_node(&schema_name_ref);
-        let name_ptr = SyntaxNodePtr::new(schema_name_ref.syntax());
+    } else if let Some(name) = create_schema.role().and_then(|role| role.name()) {
+        let schema_name = Name::from_node(&name);
+        let name_ptr = SyntaxNodePtr::new(name.syntax());
         (schema_name, name_ptr)
     } else {
         return;
@@ -278,6 +445,7 @@ fn bind_create_schema(b: &mut Binder, create_schema: ast::CreateSchema) {
         ptr: name_ptr,
         schema: Some(Schema(schema_name.clone())),
         params: None,
+        table: None,
     });
 
     let root = b.root_scope();
@@ -304,6 +472,7 @@ fn bind_create_type(b: &mut Binder, create_type: ast::CreateType) {
         ptr: name_ptr,
         schema: Some(schema),
         params: None,
+        table: None,
     });
 
     let root = b.root_scope();
@@ -331,12 +500,14 @@ fn bind_create_view(b: &mut Binder, create_view: ast::CreateView) {
         ptr: name_ptr,
         schema: Some(schema),
         params: None,
+        table: None,
     });
 
     let root = b.root_scope();
     b.scopes[root].insert(view_name, view_id);
 }
 
+// TODO: combine with create_view
 fn bind_create_materialized_view(b: &mut Binder, create_view: ast::CreateMaterializedView) {
     let Some(path) = create_view.path() else {
         return;
@@ -357,6 +528,7 @@ fn bind_create_materialized_view(b: &mut Binder, create_view: ast::CreateMateria
         ptr: name_ptr,
         schema: Some(schema),
         params: None,
+        table: None,
     });
 
     let root = b.root_scope();
@@ -385,12 +557,65 @@ fn bind_create_sequence(b: &mut Binder, create_sequence: ast::CreateSequence) {
         ptr: name_ptr,
         schema: Some(schema),
         params: None,
+        table: None,
     });
 
     let root = b.root_scope();
     b.scopes[root].insert(sequence_name, sequence_id);
 }
 
+fn bind_create_trigger(b: &mut Binder, create_trigger: ast::CreateTrigger) {
+    let Some(name) = create_trigger.name() else {
+        return;
+    };
+
+    let trigger_name = Name::from_node(&name);
+    let name_ptr = SyntaxNodePtr::new(name.syntax());
+
+    let Some(table_path) = create_trigger.on_table().and_then(|on| on.path()) else {
+        return;
+    };
+
+    let Some(table_name) = item_name(&table_path) else {
+        return;
+    };
+
+    let Some(schema) = schema_name(b, &table_path, false) else {
+        return;
+    };
+
+    let trigger_id = b.symbols.alloc(Symbol {
+        kind: SymbolKind::Trigger,
+        ptr: name_ptr,
+        schema: Some(schema),
+        params: None,
+        table: Some(table_name),
+    });
+
+    let root = b.root_scope();
+    b.scopes[root].insert(trigger_name, trigger_id);
+}
+
+fn bind_create_event_trigger(b: &mut Binder, create_event_trigger: ast::CreateEventTrigger) {
+    let Some(name) = create_event_trigger.name() else {
+        return;
+    };
+
+    let event_trigger_name = Name::from_node(&name);
+    let name_ptr = SyntaxNodePtr::new(name.syntax());
+
+    let event_trigger_id = b.symbols.alloc(Symbol {
+        kind: SymbolKind::EventTrigger,
+        ptr: name_ptr,
+        schema: None,
+        params: None,
+        table: None,
+    });
+
+    let root = b.root_scope();
+    b.scopes[root].insert(event_trigger_name, event_trigger_id);
+}
+
 fn bind_create_tablespace(b: &mut Binder, create_tablespace: ast::CreateTablespace) {
     let Some(name) = create_tablespace.name() else {
         return;
@@ -404,6 +629,7 @@ fn bind_create_tablespace(b: &mut Binder, create_tablespace: ast::CreateTablespa
         ptr: name_ptr,
         schema: None,
         params: None,
+        table: None,
     });
 
     let root = b.root_scope();
@@ -423,6 +649,7 @@ fn bind_create_database(b: &mut Binder, create_database: ast::CreateDatabase) {
         ptr: name_ptr,
         schema: None,
         params: None,
+        table: None,
     });
 
     let root = b.root_scope();
@@ -442,12 +669,113 @@ fn bind_create_server(b: &mut Binder, create_server: ast::CreateServer) {
         ptr: name_ptr,
         schema: None,
         params: None,
+        table: None,
     });
 
     let root = b.root_scope();
     b.scopes[root].insert(server_name, server_id);
 }
 
+fn bind_create_extension(b: &mut Binder, create_extension: ast::CreateExtension) {
+    let Some(name) = create_extension.name() else {
+        return;
+    };
+
+    let extension_name = Name::from_node(&name);
+    let name_ptr = SyntaxNodePtr::new(name.syntax());
+
+    let extension_id = b.symbols.alloc(Symbol {
+        kind: SymbolKind::Extension,
+        ptr: name_ptr,
+        schema: None,
+        params: None,
+        table: None,
+    });
+
+    let root = b.root_scope();
+    b.scopes[root].insert(extension_name, extension_id);
+}
+
+fn bind_create_role(b: &mut Binder, create_role: ast::CreateRole) {
+    let Some(name) = create_role.name() else {
+        return;
+    };
+
+    let role_name = Name::from_node(&name);
+    let name_ptr = SyntaxNodePtr::new(name.syntax());
+
+    let role_id = b.symbols.alloc(Symbol {
+        kind: SymbolKind::Role,
+        ptr: name_ptr,
+        schema: None,
+        params: None,
+        table: None,
+    });
+
+    let root = b.root_scope();
+    b.scopes[root].insert(role_name, role_id);
+}
+
+fn bind_declare_cursor(b: &mut Binder, declare: ast::Declare) {
+    let Some(name) = declare.name() else {
+        return;
+    };
+
+    let cursor_name = Name::from_node(&name);
+    let name_ptr = SyntaxNodePtr::new(name.syntax());
+
+    let cursor_id = b.symbols.alloc(Symbol {
+        kind: SymbolKind::Cursor,
+        ptr: name_ptr,
+        schema: None,
+        params: None,
+        table: None,
+    });
+
+    let root = b.root_scope();
+    b.scopes[root].insert(cursor_name, cursor_id);
+}
+
+fn bind_prepare(b: &mut Binder, prepare: ast::Prepare) {
+    let Some(name) = prepare.name() else {
+        return;
+    };
+
+    let statement_name = Name::from_node(&name);
+    let name_ptr = SyntaxNodePtr::new(name.syntax());
+
+    let statement_id = b.symbols.alloc(Symbol {
+        kind: SymbolKind::PreparedStatement,
+        ptr: name_ptr,
+        schema: None,
+        params: None,
+        table: None,
+    });
+
+    let root = b.root_scope();
+    b.scopes[root].insert(statement_name, statement_id);
+}
+
+fn bind_listen(b: &mut Binder, listen: ast::Listen) {
+    let Some(name) = listen.name() else {
+        return;
+    };
+
+    let channel_name = Name::from_node(&name);
+    let name_ptr = SyntaxNodePtr::new(name.syntax());
+
+    let channel_id = b.symbols.alloc(Symbol {
+        kind: SymbolKind::Channel,
+        ptr: name_ptr,
+        schema: None,
+        params: None,
+        table: None,
+    });
+
+    let root = b.root_scope();
+    b.scopes[root].insert(channel_name, channel_id);
+}
+
 fn item_name(path: &ast::Path) -> Option<Name> {
     let segment = path.segment()?;
 
@@ -529,7 +857,11 @@ fn bind_set(b: &mut Binder, set: ast::Set) {
     if set.default_token().is_some() {
         b.search_path_changes.push(SearchPathChange {
             position,
-            search_path: vec![Schema::new("public"), Schema::new("pg_temp")],
+            search_path: vec![
+                Schema::new("public"),
+                Schema::new("pg_temp"),
+                Schema::new("pg_catalog"),
+            ],
         });
     } else {
         let mut search_path = vec![];
