@@ -1,21 +1,33 @@
-use squawk_syntax::ast::{self, AstNode};
+use squawk_syntax::{
+    SyntaxKind,
+    ast::{self, AstNode},
+};
 
 #[derive(Debug)]
 pub(crate) enum NameRefClass {
     DropTable,
+    DropForeignTable,
     Table,
     DropIndex,
     DropType,
     DropView,
     DropMaterializedView,
     DropSequence,
+    DropTrigger,
+    DropEventTrigger,
     SequenceOwnedByColumn,
     Tablespace,
     DropDatabase,
     DropServer,
     AlterServer,
     CreateServer,
+    DropExtension,
+    AlterExtension,
     ForeignTableServerName,
+    AlterRole,
+    DropRole,
+    SetRole,
+    Role,
     ForeignKeyTable,
     ForeignKeyColumn,
     ForeignKeyLocalColumn,
@@ -35,7 +47,6 @@ pub(crate) enum NameRefClass {
     DropRoutine,
     CallProcedure,
     DropSchema,
-    CreateSchema,
     CreateIndex,
     CreateIndexColumn,
     DefaultConstraintFunctionCall,
@@ -85,6 +96,47 @@ pub(crate) enum NameRefClass {
     ReindexSchema,
     ReindexDatabase,
     ReindexSystem,
+    AttachPartition,
+    NamedArgParameter,
+    Cursor,
+    PreparedStatement,
+    NotifyChannel,
+    UnlistenChannel,
+    TriggerFunctionCall,
+    TriggerProcedureCall,
+    AlterEventTrigger,
+}
+
+fn is_special_fn(kind: SyntaxKind) -> bool {
+    matches!(
+        kind,
+        SyntaxKind::EXTRACT_FN
+            | SyntaxKind::JSON_EXISTS_FN
+            | SyntaxKind::JSON_ARRAY_FN
+            | SyntaxKind::JSON_OBJECT_FN
+            | SyntaxKind::JSON_OBJECT_AGG_FN
+            | SyntaxKind::JSON_ARRAY_AGG_FN
+            | SyntaxKind::JSON_QUERY_FN
+            | SyntaxKind::JSON_SCALAR_FN
+            | SyntaxKind::JSON_SERIALIZE_FN
+            | SyntaxKind::JSON_VALUE_FN
+            | SyntaxKind::JSON_FN
+            | SyntaxKind::SUBSTRING_FN
+            | SyntaxKind::POSITION_FN
+            | SyntaxKind::OVERLAY_FN
+            | SyntaxKind::TRIM_FN
+            | SyntaxKind::XML_ROOT_FN
+            | SyntaxKind::XML_SERIALIZE_FN
+            | SyntaxKind::XML_ELEMENT_FN
+            | SyntaxKind::XML_FOREST_FN
+            | SyntaxKind::XML_EXISTS_FN
+            | SyntaxKind::XML_PARSE_FN
+            | SyntaxKind::XML_PI_FN
+            | SyntaxKind::SOME_FN
+            | SyntaxKind::ANY_FN
+            | SyntaxKind::ALL_FN
+            | SyntaxKind::EXISTS_FN
+    )
 }
 
 pub(crate) fn classify_name_ref(name_ref: &ast::NameRef) -> Option<NameRefClass> {
@@ -103,6 +155,7 @@ pub(crate) fn classify_name_ref(name_ref: &ast::NameRef) -> Option<NameRefClass>
     let mut in_using_clause = false;
     let mut in_returning_clause = false;
     let mut in_when_clause = false;
+    let mut in_special_sql_fn = false;
 
     // TODO: can we combine this if and the one that follows?
     if let Some(parent) = name_ref.syntax().parent()
@@ -146,7 +199,7 @@ pub(crate) fn classify_name_ref(name_ref: &ast::NameRef) -> Option<NameRefClass>
             if ast::WhereClause::can_cast(ancestor.kind()) {
                 in_where_clause = true;
             }
-            if ast::MergeWhenMatched::can_cast(ancestor.kind()) {
+            if ast::MergeWhenClause::can_cast(ancestor.kind()) {
                 in_when_clause = true;
             }
             if ast::Update::can_cast(ancestor.kind()) {
@@ -256,7 +309,7 @@ pub(crate) fn classify_name_ref(name_ref: &ast::NameRef) -> Option<NameRefClass>
             if ast::FromClause::can_cast(ancestor.kind()) {
                 in_from_clause = true;
             }
-            if ast::MergeWhenMatched::can_cast(ancestor.kind()) {
+            if ast::MergeWhenClause::can_cast(ancestor.kind()) {
                 in_when_clause = true;
             }
             if ast::ReturningClause::can_cast(ancestor.kind()) {
@@ -303,7 +356,6 @@ pub(crate) fn classify_name_ref(name_ref: &ast::NameRef) -> Option<NameRefClass>
     }
 
     let mut in_type = false;
-    let mut in_schema_authorization = false;
     for ancestor in name_ref.syntax().ancestors() {
         if ast::PathType::can_cast(ancestor.kind()) || ast::ExprType::can_cast(ancestor.kind()) {
             in_type = true;
@@ -311,12 +363,28 @@ pub(crate) fn classify_name_ref(name_ref: &ast::NameRef) -> Option<NameRefClass>
         if in_type {
             return Some(NameRefClass::TypeReference);
         }
-        if ast::SchemaAuthorization::can_cast(ancestor.kind()) {
-            in_schema_authorization = true;
+        if ast::Fetch::can_cast(ancestor.kind())
+            || ast::Move::can_cast(ancestor.kind())
+            || ast::Close::can_cast(ancestor.kind())
+            || ast::WhereCurrentOf::can_cast(ancestor.kind())
+        {
+            return Some(NameRefClass::Cursor);
+        }
+        if ast::Execute::can_cast(ancestor.kind()) || ast::Deallocate::can_cast(ancestor.kind()) {
+            return Some(NameRefClass::PreparedStatement);
+        }
+        if ast::Notify::can_cast(ancestor.kind()) {
+            return Some(NameRefClass::NotifyChannel);
+        }
+        if ast::Unlisten::can_cast(ancestor.kind()) {
+            return Some(NameRefClass::UnlistenChannel);
         }
         if ast::DropTable::can_cast(ancestor.kind()) {
             return Some(NameRefClass::DropTable);
         }
+        if ast::DropForeignTable::can_cast(ancestor.kind()) {
+            return Some(NameRefClass::DropForeignTable);
+        }
         if ast::Truncate::can_cast(ancestor.kind()) {
             return Some(NameRefClass::TruncateTable);
         }
@@ -335,6 +403,12 @@ pub(crate) fn classify_name_ref(name_ref: &ast::NameRef) -> Option<NameRefClass>
         if ast::AlterTable::can_cast(ancestor.kind()) {
             return Some(NameRefClass::AlterTable);
         }
+        if ast::OnTable::can_cast(ancestor.kind()) {
+            return Some(NameRefClass::Table);
+        }
+        if ast::AttachPartition::can_cast(ancestor.kind()) {
+            return Some(NameRefClass::AttachPartition);
+        }
         if ast::Refresh::can_cast(ancestor.kind()) {
             return Some(NameRefClass::RefreshMaterializedView);
         }
@@ -373,6 +447,12 @@ pub(crate) fn classify_name_ref(name_ref: &ast::NameRef) -> Option<NameRefClass>
         if ast::DropSequence::can_cast(ancestor.kind()) {
             return Some(NameRefClass::DropSequence);
         }
+        if ast::DropTrigger::can_cast(ancestor.kind()) {
+            return Some(NameRefClass::DropTrigger);
+        }
+        if ast::DropEventTrigger::can_cast(ancestor.kind()) {
+            return Some(NameRefClass::DropEventTrigger);
+        }
         if ast::DropDatabase::can_cast(ancestor.kind()) {
             return Some(NameRefClass::DropDatabase);
         }
@@ -382,9 +462,30 @@ pub(crate) fn classify_name_ref(name_ref: &ast::NameRef) -> Option<NameRefClass>
         if ast::AlterServer::can_cast(ancestor.kind()) {
             return Some(NameRefClass::AlterServer);
         }
+        if ast::AlterEventTrigger::can_cast(ancestor.kind()) {
+            return Some(NameRefClass::AlterEventTrigger);
+        }
         if ast::CreateServer::can_cast(ancestor.kind()) {
             return Some(NameRefClass::CreateServer);
         }
+        if ast::DropExtension::can_cast(ancestor.kind()) {
+            return Some(NameRefClass::DropExtension);
+        }
+        if ast::AlterExtension::can_cast(ancestor.kind()) {
+            return Some(NameRefClass::AlterExtension);
+        }
+        if ast::AlterRole::can_cast(ancestor.kind()) {
+            return Some(NameRefClass::AlterRole);
+        }
+        if ast::DropRole::can_cast(ancestor.kind()) {
+            return Some(NameRefClass::DropRole);
+        }
+        if ast::SetRole::can_cast(ancestor.kind()) {
+            return Some(NameRefClass::SetRole);
+        }
+        if ast::RoleRef::can_cast(ancestor.kind()) {
+            return Some(NameRefClass::Role);
+        }
         if ast::ServerName::can_cast(ancestor.kind()) {
             return Some(NameRefClass::ForeignTableServerName);
         }
@@ -500,24 +601,42 @@ pub(crate) fn classify_name_ref(name_ref: &ast::NameRef) -> Option<NameRefClass>
         if ast::DropSchema::can_cast(ancestor.kind()) {
             return Some(NameRefClass::DropSchema);
         }
-        if in_schema_authorization
-            && let Some(create_schema) = ast::CreateSchema::cast(ancestor.clone())
-            && create_schema.name().is_none()
-        {
-            return Some(NameRefClass::CreateSchema);
-        }
         if ast::CreateIndex::can_cast(ancestor.kind()) {
             if in_partition_item {
                 return Some(NameRefClass::CreateIndexColumn);
             }
             return Some(NameRefClass::CreateIndex);
         }
+        if let Some(create_trigger) = ast::CreateTrigger::cast(ancestor.clone())
+            && in_call_expr
+            && !in_arg_list
+        {
+            if create_trigger.procedure_token().is_some() {
+                return Some(NameRefClass::TriggerProcedureCall);
+            }
+            return Some(NameRefClass::TriggerFunctionCall);
+        }
+        if let Some(create_event_trigger) = ast::CreateEventTrigger::cast(ancestor.clone())
+            && in_call_expr
+            && !in_arg_list
+        {
+            if create_event_trigger.procedure_token().is_some() {
+                return Some(NameRefClass::TriggerProcedureCall);
+            }
+            return Some(NameRefClass::TriggerFunctionCall);
+        }
         if in_partition_item && ast::CreateTableLike::can_cast(ancestor.kind()) {
             return Some(NameRefClass::PartitionByColumn);
         }
         if ast::PartitionOf::can_cast(ancestor.kind()) {
             return Some(NameRefClass::PartitionOfTable);
         }
+        if is_special_fn(ancestor.kind()) {
+            in_special_sql_fn = true;
+        }
+        if ast::NamedArg::can_cast(ancestor.kind()) {
+            return Some(NameRefClass::NamedArgParameter);
+        }
         if ast::ArgList::can_cast(ancestor.kind()) {
             in_arg_list = true;
         }
@@ -534,7 +653,7 @@ pub(crate) fn classify_name_ref(name_ref: &ast::NameRef) -> Option<NameRefClass>
             in_from_clause = true;
         }
         if ast::Select::can_cast(ancestor.kind()) {
-            if in_call_expr && !in_arg_list {
+            if in_call_expr && !in_arg_list && !in_special_sql_fn {
                 return Some(NameRefClass::SelectFunctionCall);
             }
             if in_from_clause && !in_on_clause {
@@ -613,7 +732,7 @@ pub(crate) fn classify_name_ref(name_ref: &ast::NameRef) -> Option<NameRefClass>
             }
             return Some(NameRefClass::UpdateTable);
         }
-        if ast::MergeWhenMatched::can_cast(ancestor.kind()) {
+        if ast::MergeWhenClause::can_cast(ancestor.kind()) {
             in_when_clause = true;
         }
         if ast::Merge::can_cast(ancestor.kind()) {
@@ -646,9 +765,13 @@ pub(crate) enum NameClass {
     WithTable(ast::WithTable),
     CreateIndex(ast::CreateIndex),
     CreateSequence(ast::CreateSequence),
+    CreateTrigger(ast::CreateTrigger),
+    CreateEventTrigger(ast::CreateEventTrigger),
     CreateTablespace(ast::CreateTablespace),
     CreateDatabase(ast::CreateDatabase),
     CreateServer(ast::CreateServer),
+    CreateExtension(ast::CreateExtension),
+    CreateRole(ast::CreateRole),
     CreateType(ast::CreateType),
     CreateFunction(ast::CreateFunction),
     CreateAggregate(ast::CreateAggregate),
@@ -659,6 +782,9 @@ pub(crate) enum NameClass {
         name: ast::Name,
     },
     CreateView(ast::CreateView),
+    DeclareCursor(ast::Declare),
+    PrepareStatement(ast::Prepare),
+    Listen(ast::Listen),
 }
 
 pub(crate) fn classify_name(name: &ast::Name) -> Option<NameClass> {
@@ -686,6 +812,12 @@ pub(crate) fn classify_name(name: &ast::Name) -> Option<NameClass> {
         if let Some(create_sequence) = ast::CreateSequence::cast(ancestor.clone()) {
             return Some(NameClass::CreateSequence(create_sequence));
         }
+        if let Some(create_trigger) = ast::CreateTrigger::cast(ancestor.clone()) {
+            return Some(NameClass::CreateTrigger(create_trigger));
+        }
+        if let Some(create_event_trigger) = ast::CreateEventTrigger::cast(ancestor.clone()) {
+            return Some(NameClass::CreateEventTrigger(create_event_trigger));
+        }
         if let Some(create_tablespace) = ast::CreateTablespace::cast(ancestor.clone()) {
             return Some(NameClass::CreateTablespace(create_tablespace));
         }
@@ -695,6 +827,12 @@ pub(crate) fn classify_name(name: &ast::Name) -> Option<NameClass> {
         if let Some(create_server) = ast::CreateServer::cast(ancestor.clone()) {
             return Some(NameClass::CreateServer(create_server));
         }
+        if let Some(create_extension) = ast::CreateExtension::cast(ancestor.clone()) {
+            return Some(NameClass::CreateExtension(create_extension));
+        }
+        if let Some(create_role) = ast::CreateRole::cast(ancestor.clone()) {
+            return Some(NameClass::CreateRole(create_role));
+        }
         if let Some(create_type) = ast::CreateType::cast(ancestor.clone()) {
             return Some(NameClass::CreateType(create_type));
         }
@@ -719,6 +857,15 @@ pub(crate) fn classify_name(name: &ast::Name) -> Option<NameClass> {
             }
             return Some(NameClass::CreateView(create_view));
         }
+        if let Some(declare) = ast::Declare::cast(ancestor.clone()) {
+            return Some(NameClass::DeclareCursor(declare));
+        }
+        if let Some(prepare) = ast::Prepare::cast(ancestor.clone()) {
+            return Some(NameClass::PrepareStatement(prepare));
+        }
+        if let Some(listen) = ast::Listen::cast(ancestor.clone()) {
+            return Some(NameClass::Listen(listen));
+        }
     }
 
     if let Some(with_table) = with_table_parent {
@@ -727,3 +874,17 @@ pub(crate) fn classify_name(name: &ast::Name) -> Option<NameClass> {
 
     None
 }
+
+#[test]
+fn special_function() {
+    for kind in (0..SyntaxKind::__LAST as u16)
+        .map(SyntaxKind::from)
+        .filter(|kind| format!("{:?}", kind).ends_with("_FN"))
+    {
+        assert!(
+            is_special_fn(kind),
+            "unhandled special function kind: {:?}. Please update is_special_fn",
+            kind
+        )
+    }
+}
