@@ -3,8 +3,8 @@ use squawk_syntax::ast::{self, AstNode};
 
 use crate::binder::{self, extract_string_literal};
 use crate::resolve::{
-    resolve_aggregate_info, resolve_function_info, resolve_materialized_view_info,
-    resolve_procedure_info, resolve_table_info, resolve_type_info, resolve_view_info,
+    resolve_aggregate_info, resolve_function_info, resolve_procedure_info, resolve_table_info,
+    resolve_type_info, resolve_view_info,
 };
 
 #[derive(Debug)]
@@ -16,10 +16,15 @@ pub enum DocumentSymbolKind {
     Function,
     Aggregate,
     Procedure,
+    EventTrigger,
+    Role,
     Type,
     Enum,
     Column,
     Variant,
+    Cursor,
+    PreparedStatement,
+    Channel,
 }
 
 #[derive(Debug)]
@@ -66,6 +71,16 @@ pub fn document_symbols(file: &ast::SourceFile) -> Vec<DocumentSymbol> {
                     symbols.push(symbol);
                 }
             }
+            ast::Stmt::CreateEventTrigger(create_event_trigger) => {
+                if let Some(symbol) = create_event_trigger_symbol(create_event_trigger) {
+                    symbols.push(symbol);
+                }
+            }
+            ast::Stmt::CreateRole(create_role) => {
+                if let Some(symbol) = create_role_symbol(create_role) {
+                    symbols.push(symbol);
+                }
+            }
             ast::Stmt::CreateType(create_type) => {
                 if let Some(symbol) = create_type_symbol(&binder, create_type) {
                     symbols.push(symbol);
@@ -81,6 +96,16 @@ pub fn document_symbols(file: &ast::SourceFile) -> Vec<DocumentSymbol> {
                     symbols.push(symbol);
                 }
             }
+            ast::Stmt::Declare(declare) => {
+                if let Some(symbol) = create_declare_cursor_symbol(declare) {
+                    symbols.push(symbol);
+                }
+            }
+            ast::Stmt::Prepare(prepare) => {
+                if let Some(symbol) = create_prepare_symbol(prepare) {
+                    symbols.push(symbol);
+                }
+            }
             ast::Stmt::Select(select) => {
                 symbols.extend(cte_table_symbols(select));
             }
@@ -96,6 +121,21 @@ pub fn document_symbols(file: &ast::SourceFile) -> Vec<DocumentSymbol> {
             ast::Stmt::Delete(delete) => {
                 symbols.extend(cte_table_symbols(delete));
             }
+            ast::Stmt::Listen(listen) => {
+                if let Some(symbol) = create_listen_symbol(listen) {
+                    symbols.push(symbol);
+                }
+            }
+            ast::Stmt::Notify(notify) => {
+                if let Some(symbol) = create_notify_symbol(notify) {
+                    symbols.push(symbol);
+                }
+            }
+            ast::Stmt::Unlisten(unlisten) => {
+                if let Some(symbol) = create_unlisten_symbol(unlisten) {
+                    symbols.push(symbol);
+                }
+            }
 
             _ => {}
         }
@@ -122,23 +162,13 @@ fn create_cte_table_symbol(with_table: ast::WithTable) -> Option<DocumentSymbol>
     let full_range = with_table.syntax().text_range();
     let focus_range = name_node.syntax().text_range();
 
-    let mut children = vec![];
-    if let Some(column_list) = with_table.column_list() {
-        for column in column_list.columns() {
-            if let Some(column_symbol) = create_column_symbol(column) {
-                children.push(column_symbol);
-            }
-        }
-    }
-
-    Some(DocumentSymbol {
+    symbols_from_column_list(
+        with_table.column_list(),
         name,
-        detail: None,
-        kind: DocumentSymbolKind::Table,
         full_range,
         focus_range,
-        children,
-    })
+        DocumentSymbolKind::Table,
+    )
 }
 
 fn create_schema_symbol(create_schema: ast::CreateSchema) -> Option<DocumentSymbol> {
@@ -147,15 +177,8 @@ fn create_schema_symbol(create_schema: ast::CreateSchema) -> Option<DocumentSymb
             name_node.syntax().text().to_string(),
             name_node.syntax().text_range(),
         )
-    } else if let Some(schema_name_ref) = create_schema
-        .schema_authorization()
-        .and_then(|authorization| authorization.role())
-        .and_then(|role| role.name_ref())
-    {
-        (
-            schema_name_ref.syntax().text().to_string(),
-            schema_name_ref.syntax().text_range(),
-        )
+    } else if let Some(name) = create_schema.role().and_then(|r| r.name()) {
+        (name.syntax().text().to_string(), name.syntax().text_range())
     } else {
         return None;
     };
@@ -221,8 +244,24 @@ fn create_view_symbol(
     let full_range = create_view.syntax().text_range();
     let focus_range = name_node.syntax().text_range();
 
+    symbols_from_column_list(
+        create_view.column_list(),
+        name,
+        full_range,
+        focus_range,
+        DocumentSymbolKind::View,
+    )
+}
+
+fn symbols_from_column_list(
+    column_list: Option<ast::ColumnList>,
+    name: String,
+    full_range: TextRange,
+    focus_range: TextRange,
+    kind: DocumentSymbolKind,
+) -> Option<DocumentSymbol> {
     let mut children = vec![];
-    if let Some(column_list) = create_view.column_list() {
+    if let Some(column_list) = column_list {
         for column in column_list.columns() {
             if let Some(column_symbol) = create_column_symbol(column) {
                 children.push(column_symbol);
@@ -233,13 +272,14 @@ fn create_view_symbol(
     Some(DocumentSymbol {
         name,
         detail: None,
-        kind: DocumentSymbolKind::View,
+        kind,
         full_range,
         focus_range,
         children,
     })
 }
 
+// TODO: combine with create_view_symbol
 fn create_materialized_view_symbol(
     binder: &binder::Binder,
     create_view: ast::CreateMaterializedView,
@@ -248,29 +288,19 @@ fn create_materialized_view_symbol(
     let segment = path.segment()?;
     let name_node = segment.name()?;
 
-    let (schema, view_name) = resolve_materialized_view_info(binder, &path)?;
+    let (schema, view_name) = resolve_view_info(binder, &path)?;
     let name = format!("{}.{}", schema.0, view_name);
 
     let full_range = create_view.syntax().text_range();
     let focus_range = name_node.syntax().text_range();
 
-    let mut children = vec![];
-    if let Some(column_list) = create_view.column_list() {
-        for column in column_list.columns() {
-            if let Some(column_symbol) = create_column_symbol(column) {
-                children.push(column_symbol);
-            }
-        }
-    }
-
-    Some(DocumentSymbol {
+    symbols_from_column_list(
+        create_view.column_list(),
         name,
-        detail: None,
-        kind: DocumentSymbolKind::MaterializedView,
         full_range,
         focus_range,
-        children,
-    })
+        DocumentSymbolKind::MaterializedView,
+    )
 }
 
 fn create_function_symbol(
@@ -345,6 +375,42 @@ fn create_procedure_symbol(
     })
 }
 
+fn create_event_trigger_symbol(
+    create_event_trigger: ast::CreateEventTrigger,
+) -> Option<DocumentSymbol> {
+    let name_node = create_event_trigger.name()?;
+    let name = name_node.syntax().text().to_string();
+
+    let full_range = create_event_trigger.syntax().text_range();
+    let focus_range = name_node.syntax().text_range();
+
+    Some(DocumentSymbol {
+        name,
+        detail: None,
+        kind: DocumentSymbolKind::EventTrigger,
+        full_range,
+        focus_range,
+        children: vec![],
+    })
+}
+
+fn create_role_symbol(create_role: ast::CreateRole) -> Option<DocumentSymbol> {
+    let name_node = create_role.name()?;
+    let name = name_node.syntax().text().to_string();
+
+    let full_range = create_role.syntax().text_range();
+    let focus_range = name_node.syntax().text_range();
+
+    Some(DocumentSymbol {
+        name,
+        detail: None,
+        kind: DocumentSymbolKind::Role,
+        full_range,
+        focus_range,
+        children: vec![],
+    })
+}
+
 fn create_type_symbol(
     binder: &binder::Binder,
     create_type: ast::CreateType,
@@ -424,6 +490,91 @@ fn create_variant_symbol(variant: ast::Variant) -> Option<DocumentSymbol> {
     })
 }
 
+fn create_declare_cursor_symbol(declare: ast::Declare) -> Option<DocumentSymbol> {
+    let name_node = declare.name()?;
+    let name = name_node.syntax().text().to_string();
+
+    let full_range = declare.syntax().text_range();
+    let focus_range = name_node.syntax().text_range();
+
+    Some(DocumentSymbol {
+        name,
+        detail: None,
+        kind: DocumentSymbolKind::Cursor,
+        full_range,
+        focus_range,
+        children: vec![],
+    })
+}
+
+fn create_prepare_symbol(prepare: ast::Prepare) -> Option<DocumentSymbol> {
+    let name_node = prepare.name()?;
+    let name = name_node.syntax().text().to_string();
+
+    let full_range = prepare.syntax().text_range();
+    let focus_range = name_node.syntax().text_range();
+
+    Some(DocumentSymbol {
+        name,
+        detail: None,
+        kind: DocumentSymbolKind::PreparedStatement,
+        full_range,
+        focus_range,
+        children: vec![],
+    })
+}
+
+fn create_listen_symbol(listen: ast::Listen) -> Option<DocumentSymbol> {
+    let name_node = listen.name()?;
+    let name = name_node.syntax().text().to_string();
+
+    let full_range = listen.syntax().text_range();
+    let focus_range = name_node.syntax().text_range();
+
+    Some(DocumentSymbol {
+        name,
+        detail: Some("listen".to_string()),
+        kind: DocumentSymbolKind::Channel,
+        full_range,
+        focus_range,
+        children: vec![],
+    })
+}
+
+fn create_notify_symbol(notify: ast::Notify) -> Option<DocumentSymbol> {
+    let name_node = notify.name_ref()?;
+    let name = name_node.syntax().text().to_string();
+
+    let full_range = notify.syntax().text_range();
+    let focus_range = name_node.syntax().text_range();
+
+    Some(DocumentSymbol {
+        name,
+        detail: Some("notify".to_string()),
+        kind: DocumentSymbolKind::Channel,
+        full_range,
+        focus_range,
+        children: vec![],
+    })
+}
+
+fn create_unlisten_symbol(unlisten: ast::Unlisten) -> Option<DocumentSymbol> {
+    let name_node = unlisten.name_ref()?;
+    let name = name_node.syntax().text().to_string();
+
+    let full_range = unlisten.syntax().text_range();
+    let focus_range = name_node.syntax().text_range();
+
+    Some(DocumentSymbol {
+        name,
+        detail: Some("unlisten".to_string()),
+        kind: DocumentSymbolKind::Channel,
+        full_range,
+        focus_range,
+        children: vec![],
+    })
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -469,10 +620,15 @@ mod tests {
             DocumentSymbolKind::Function => "function",
             DocumentSymbolKind::Aggregate => "aggregate",
             DocumentSymbolKind::Procedure => "procedure",
+            DocumentSymbolKind::EventTrigger => "event trigger",
+            DocumentSymbolKind::Role => "role",
             DocumentSymbolKind::Type => "type",
             DocumentSymbolKind::Enum => "enum",
             DocumentSymbolKind::Column => "column",
             DocumentSymbolKind::Variant => "variant",
+            DocumentSymbolKind::Cursor => "cursor",
+            DocumentSymbolKind::PreparedStatement => "prepared statement",
+            DocumentSymbolKind::Channel => "channel",
         };
 
         let title = if let Some(detail) = &symbol.detail {
@@ -598,6 +754,39 @@ create schema authorization foo;
         ");
     }
 
+    #[test]
+    fn listen_notify_unlisten() {
+        assert_snapshot!(symbols("
+listen updates;
+notify updates;
+unlisten updates;
+unlisten *;
+"), @r"
+        info: channel: updates listen
+          ╭▸ 
+        2 │ listen updates;
+          │ ┬──────┯━━━━━━
+          │ │      │
+          │ │      focus range
+          │ full range
+          ╰╴
+        info: channel: updates notify
+          ╭▸ 
+        3 │ notify updates;
+          │ ┬──────┯━━━━━━
+          │ │      │
+          │ │      focus range
+          ╰╴full range
+        info: channel: updates unlisten
+          ╭▸ 
+        4 │ unlisten updates;
+          │ ┬────────┯━━━━━━
+          │ │        │
+          │ │        focus range
+          ╰╴full range
+        ");
+    }
+
     #[test]
     fn create_function() {
         assert_snapshot!(
@@ -662,6 +851,37 @@ create schema authorization foo;
         );
     }
 
+    #[test]
+    fn create_event_trigger() {
+        assert_snapshot!(
+            symbols("create event trigger et on ddl_command_start execute function f();"),
+            @r"
+        info: event trigger: et
+          ╭▸ 
+        1 │ create event trigger et on ddl_command_start execute function f();
+          │ ┬────────────────────┯━──────────────────────────────────────────
+          │ │                    │
+          │ │                    focus range
+          ╰╴full range
+        "
+        );
+    }
+
+    #[test]
+    fn create_role() {
+        assert_snapshot!(symbols("
+create role reader;
+"), @r"
+        info: role: reader
+          ╭▸ 
+        2 │ create role reader;
+          │ ┬───────────┯━━━━━
+          │ │           │
+          │ │           focus range
+          ╰╴full range
+        ");
+    }
+
     #[test]
     fn multiple_symbols() {
         assert_snapshot!(symbols("
@@ -870,6 +1090,36 @@ create function my_schema.hello() returns void as $$ select 1; $$ language sql;
         );
     }
 
+    #[test]
+    fn declare_cursor() {
+        assert_snapshot!(symbols("
+declare c scroll cursor for select * from t;
+"), @r"
+        info: cursor: c
+          ╭▸ 
+        2 │ declare c scroll cursor for select * from t;
+          │ ┬───────┯──────────────────────────────────
+          │ │       │
+          │ │       focus range
+          ╰╴full range
+        ");
+    }
+
+    #[test]
+    fn prepare_statement() {
+        assert_snapshot!(symbols("
+prepare stmt as select 1;
+"), @r"
+        info: prepared statement: stmt
+          ╭▸ 
+        2 │ prepare stmt as select 1;
+          │ ┬───────┯━━━────────────
+          │ │       │
+          │ │       focus range
+          ╰╴full range
+        ");
+    }
+
     #[test]
     fn empty_file() {
         symbols_not_found("")
