@@ -2,6 +2,7 @@ mod binder;
 mod classify;
 pub mod code_actions;
 pub mod column_name;
+pub mod completion;
 pub mod document_symbols;
 pub mod expand_selection;
 pub mod find_references;
@@ -16,3 +17,4 @@ mod scope;
 mod symbols;
 #[cfg(test)]
 pub mod test_utils;
+mod tokens;
