@@ -53,9 +53,16 @@ pub(crate) enum SymbolKind {
     Type,
     View,
     Sequence,
+    Cursor,
+    PreparedStatement,
+    Channel,
     Tablespace,
     Database,
     Server,
+    Extension,
+    Trigger,
+    EventTrigger,
+    Role,
 }
 
 #[derive(Clone, Debug)]
@@ -64,6 +71,7 @@ pub(crate) struct Symbol {
     pub(crate) ptr: SyntaxNodePtr,
     pub(crate) schema: Option<Schema>,
     pub(crate) params: Option<Vec<Name>>,
+    pub(crate) table: Option<Name>,
 }
 
 pub(crate) type SymbolId = Idx<Symbol>;
